"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const resolve_workspace_range_1 = require("@pnpm/resolve-workspace-range");
const npa = require("@zkochan/npm-package-arg");
const path = require("path");
const R = require("ramda");
function default_1(pkgs, opts) {
    const pkgMap = createPkgMap(pkgs);
    const unmatched = [];
    const graph = Object.keys(pkgMap)
        .reduce((acc, pkgSpec) => {
        acc[pkgSpec] = {
            dependencies: createNode(pkgMap[pkgSpec]),
            package: pkgMap[pkgSpec],
        };
        return acc;
    }, {});
    return { graph, unmatched };
    function createNode(pkg) {
        const dependencies = Object.assign({}, pkg.manifest.devDependencies, pkg.manifest.optionalDependencies, pkg.manifest.dependencies);
        return Object.keys(dependencies)
            .map(depName => {
            let spec;
            let rawSpec = dependencies[depName];
            const isWorkspaceSpec = rawSpec.startsWith('workspace:');
            try {
                if (isWorkspaceSpec) {
                    rawSpec = rawSpec.substr(10);
                }
                spec = npa.resolve(depName, rawSpec, pkg.dir);
            }
            catch (err) {
                return '';
            }
            if (spec.type === 'directory') {
                const matchedPkg = R.values(pkgMap).find(pkg => path.relative(pkg.dir, spec.fetchSpec) === '');
                if (!matchedPkg) {
                    return '';
                }
                return matchedPkg.dir;
            }
            if (spec.type !== 'version' && spec.type !== 'range')
                return '';
            const pkgs = R.values(pkgMap).filter(pkg => pkg.manifest.name === depName);
            if (!pkgs.length)
                return '';
            const versions = pkgs.filter(({ manifest }) => manifest.version)
                .map(pkg => pkg.manifest.version);
            // explicitly check if false, backwards-compatibility (can be undefined)
            const strictWorkspaceMatching = (opts === null || opts === void 0 ? void 0 : opts.linkWorkspacePackages) === false && !isWorkspaceSpec;
            if (strictWorkspaceMatching) {
                unmatched.push({ pkgName: depName, range: rawSpec });
                return '';
            }
            if (versions.includes(rawSpec)) {
                const matchedPkg = pkgs.find(pkg => pkg.manifest.name === depName && pkg.manifest.version === rawSpec);
                return matchedPkg.dir;
            }
            const matched = resolve_workspace_range_1.default(rawSpec, versions);
            if (!matched) {
                unmatched.push({ pkgName: depName, range: rawSpec });
                return '';
            }
            const matchedPkg = pkgs.find(pkg => pkg.manifest.name === depName && pkg.manifest.version === matched);
            return matchedPkg.dir;
        })
            .filter(Boolean);
    }
}
exports.default = default_1;
function createPkgMap(pkgs) {
    const pkgMap = {};
    for (let pkg of pkgs) {
        pkgMap[pkg.dir] = pkg;
    }
    return pkgMap;
}
//# sourceMappingURL=index.js.map