'use strict'
const path = require('path')
const fs = require('graceful-fs')
const _writeFileAtomic = require('write-file-atomic')
const YAML = require('js-yaml')
const { promisify } = require('util')

const writeFileAtomic = promisify(_writeFileAtomic)
const mkdir = promisify(fs.mkdir)

const main = (fn, fp, data, opts) => {
  if (!fp) {
    throw new TypeError('Expected a filepath')
  }

  if (data === undefined) {
    throw new TypeError('Expected data to stringify')
  }

  opts = opts || {}

  const yaml = YAML.safeDump(data, opts)

  return fn(fp, yaml, { mode: opts.mode })
}

module.exports = async (fp, data, opts) => {
  await mkdir(path.dirname(fp), { recursive: true })
  return main(writeFileAtomic, fp, data, opts)
}

module.exports.sync = (fp, data, opts) => {
  fs.mkdirSync(path.dirname(fp), { recursive: true })
  main(writeFileAtomic.sync, fp, data, opts)
}
